@@ -12,6 +12,7 @@ from sys import stderr
 try:
     import PIL
     from PIL import Image
+    from PIL.TiffImagePlugin import IFDRational
 except ImportError:
     PIL = None
 
@@ -72,7 +73,7 @@ class WDFReader(object):
                         # TODO types?
     """
 
-    def __init__(self, file_name, quiet=True):
+    def __init__(self, file_name, debug=False):
         try:
             self.file_obj = open(str(file_name), "rb")
         except IOError:
@@ -99,6 +100,7 @@ class WDFReader(object):
         self.accumulation_count = None
         self.block_info = {}    # each key has value (uid, offset, size)
         self.is_completed = False
+        self.debug = debug
         # Parse the header section in the wdf file
         self.__locate_all_blocks()
         # Parse individual blocks
@@ -115,17 +117,21 @@ class WDFReader(object):
         # self._parse_wmap()
 
         # Finally print the information
-        if not quiet:
-            self.print_info()
+        if self.debug:
+            print(("File Metadata").center(80, "="),
+                  file=stderr)
+            self.print_info(file=stderr)
+            print("=" * 80, file=stderr)
 
     def close(self):
         self.file_obj.close()
+        if hasattr(self, "img"):
+            self.img.close()
 
     def __get_type_string(self, attr, data_type):
         """Get the enumerated-data_type as string
         """
         val = getattr(self, attr)  # No error checking
-        # print(attr, data_type, val)
         if data_type is None:
             return val
         else:
@@ -168,7 +174,6 @@ class WDFReader(object):
             try:
                 block_name, block_uid, block_size = self.__locate_single_block(
                     curpos)
-                # print(block_name, block_uid, block_size)
                 self.block_info[block_name] = (block_uid, curpos, block_size)
                 curpos += block_size
             except (EOFError, UnicodeDecodeError):
@@ -178,8 +183,9 @@ class WDFReader(object):
         """Get data according to specific block name
         """
         if block_name not in self.block_info.keys():
-            print("Block name {0} not present in current measurement".
-                  format(block_name), file=stderr)
+            if self.debug:
+                print("Block name {0} not present in current measurement".
+                      format(block_name), file=stderr)
             return
         # parse individual blocks with names
         actions = {
@@ -345,8 +351,10 @@ class WDFReader(object):
         try:
             uid, pos, size = self.block_info["WMAP"]
         except KeyError:
-            print("Current measurement does not contain mapping information!",
-                  file=stderr)
+            if self.debug:
+                print(("Current measurement does not"
+                       " contain mapping information!"),
+                      file=stderr)
             return
 
         self.file_obj.seek(pos + Offsets.wmap_origin)
@@ -360,15 +368,9 @@ class WDFReader(object):
         x_pad = self.__read_type("float")
         y_pad = self.__read_type("float")
         unknown2 = self.__read_type("float")
-        # for i in range(9):
-        #     print(i, self._read_type("int32"))
-        #     self.file_obj.seek(pos + Offsets.wmap_origin + i * 4)
-        #     print(i, self._read_type("float"))
-        # self.file_obj.seek(pos + Offsets.wmap_wh)
         spectra_w = self.__read_type("int32")
         spectra_h = self.__read_type("int32")
-        # print(self.spectra_w, self.spectra_h)
-        # print(len(self.xpos), len(self.ypos))
+
         if len(self.xpos) > 1:
             if not numpy.isclose(x_pad, self.xpos[1] - self.xpos[0],
                                  rtol=1e-4):
@@ -401,8 +403,9 @@ class WDFReader(object):
         try:
             uid, pos, size = self.block_info["WHTL"]
         except KeyError:
-            print("The wdf file does not contain an image",
-                  file=stderr)
+            if self.debug:
+                print("The wdf file does not contain an image",
+                      file=stderr)
             return
 
         # Read the bytes. `self.img` is a wrapped IO object mimicking a file
@@ -418,13 +421,23 @@ class WDFReader(object):
                 w_ = exif_header[ExifTags.FocalPlaneXResolution]
                 h_ = exif_header[ExifTags.FocalPlaneYResolution]
                 x_org_, y_org_ = exif_header[ExifTags.FocalPlaneXYOrigins]
+
+                def rational2float(v):
+                    """ Pillow<7.2.0 returns tuple, Pillow>=7.2.0 returns IFDRational """
+                    if not isinstance(v, IFDRational):
+                        return v[0] / v[1]
+                    return float(v)
+
+                w_, h_ = rational2float(w_), rational2float(h_)
+                x_org_, y_org_ = rational2float(x_org_), rational2float(y_org_)
+
                 # The dimensions (width, height)
                 # with unit `img_dimension_unit`
-                self.img_dimensions = numpy.array([w_[0] / w_[1],
-                                                   h_[0] / h_[1]])
+                self.img_dimensions = numpy.array([w_,
+                                                   h_])
                 # Origin of image is at upper right corner
-                self.img_origins = numpy.array([x_org_[0] / x_org_[1],
-                                                y_org_[0] / y_org_[1]])
+                self.img_origins = numpy.array([x_org_,
+                                                y_org_])
                 # Default is microns (5)
                 self.img_dimension_unit = UnitType(
                     exif_header[ExifTags.FocalPlaneResolutionUnit])
@@ -434,8 +447,10 @@ class WDFReader(object):
                 self.img_cropbox = self.__calc_crop_box()
 
             except KeyError:
-                print("Some keys in white light image header cannot be read!",
-                      file=stderr)
+                if self.debug:
+                    print(("Some keys in white light image header"
+                           " cannot be read!"),
+                          file=stderr)
         return
 
     def __calc_crop_box(self):
@@ -465,27 +480,36 @@ class WDFReader(object):
         """Reshape spectra into w * h * self.point_per_spectrum
         """
         if not self.is_completed:
-            print("The measurement is not completed, " +
-                  "will try to reshape spectra into count * pps.", file=stderr)
+            if self.debug:
+                print(("The measurement is not completed, "
+                       "will try to reshape spectra into count * pps."),
+                      file=stderr)
             try:
                 self.spectra = numpy.reshape(self.spectra,
                                              (self.count,
                                               self.point_per_spectrum))
             except ValueError:
-                print("Reshaping spectra array failed. Please check.",
-                      file=stderr)
+                if self.debug:
+                    print("Reshaping spectra array failed. Please check.",
+                          file=stderr)
             return
         elif hasattr(self, "map_shape"):
             # Is a mapping
             spectra_w, spectra_h = self.map_shape
             if spectra_w * spectra_h != self.count:
-                print("Mapping information from WMAP not corresponding to ORGN! " +
-                      "Will not reshape the spectra", file=stderr)
+                if self.debug:
+                    print(("Mapping information from WMAP not"
+                           " corresponding to ORGN! "
+                           "Will not reshape the spectra"),
+                          file=stderr)
                 return
             elif spectra_w * spectra_h * self.point_per_spectrum \
                     != len(self.spectra):
-                print("Mapping information from WMAP not corresponding to DATA! " +
-                      "Will not reshape the spectra", file=stderr)
+                if self.debug:
+                    print(("Mapping information from WMAP"
+                           " not corresponding to DATA! "
+                           "Will not reshape the spectra"),
+                          file=stderr)
                 return
             else:
                 # Should be h rows * w columns. numpy.ndarray is row first
@@ -508,7 +532,7 @@ class WDFReader(object):
         else:
             return
 
-    def print_info(self):
+    def print_info(self, **params):
         """Print information of the wdf file
         """
         s = []
@@ -532,28 +556,8 @@ class WDFReader(object):
             except AttributeError:
                 continue
             s.append("{0:>24s}:\t{1}".format(sname, val))
-        print("\n".join(s))
+        print("\n".join(s), **params)
 
 
 if __name__ == '__main__':
-    import sys
-    try:
-        fn = sys.argv[1]
-        # print(fn)
-        wdf = WDFReader(fn)
-        # for s in dir(wdf):
-        # if "__" not in s:
-        # print(s, getattr(wdf, s))
-        wdf.print_info()
-        print(wdf.spectra.shape)
-        # print(wdf.spectra_w, wdf.spectra_h)
-        # xdata = wdf.get_xdata()
-        # ydata = wdf.get_ydata()
-        # spectra = wdf.get_spectra()
-        # print(wdf.xdata)
-        # print(wdf.ydata)
-        # print(wdf.spectra)
-    except IndexError:
-        raise
-    # plt.plot(xdata, spectra)
-    # plt.show()
+    raise NotImplementedError("Please dont run this module as a script!")
